<?php

namespace IQBall\App\Controller;

use IQBall\App\Session\MutableSessionHandle;
use IQBall\App\ViewHttpResponse;
use IQBall\Core\Http\HttpRequest;
use IQBall\Core\Http\HttpResponse;
use IQBall\Core\Model\AuthModel;

use IQBall\Core\Validation\DefaultValidators;
use IQBall\Core\Validation\FieldValidationFail;

class AuthController {
    private AuthModel $model;

    /**
     * @param AuthModel $model
     */
    public function __construct(AuthModel $model) {
        $this->model = $model;
    }

    public function displayRegister(): HttpResponse {
        return ViewHttpResponse::twig("display_register.html.twig", []);
    }

    /**
     * registers given account
     * @param mixed[] $requestData
     * @param MutableSessionHandle $session
     * @return HttpResponse
     */
    public function register(array $requestData, MutableSessionHandle $session): HttpResponse {
        $fails = [];
        $request = HttpRequest::from($requestData, $fails, [
            "username" => [DefaultValidators::name(), DefaultValidators::lenBetween(2, 32)],
            "password" => [DefaultValidators::password()],
            "confirmpassword" => [DefaultValidators::password()],
            "email" => [DefaultValidators::email(), DefaultValidators::lenBetween(5, 256)],
        ]);

        if (!empty($fails)) {
            return ViewHttpResponse::twig("display_register.html.twig", ['fails' => $fails, 'username' => $requestData['username'], 'email' => $requestData['email'], 'password' => $requestData['password'], 'confirmpassword' => $requestData['confirmpassword'], 'accept' => true]);
        }

        if ($request["password"] != $request['confirmpassword']) {
            $fails[] = new FieldValidationFail("confirmpassword", "Le mot de passe et la confirmation ne sont pas les mêmes.");
        }

        $account = $this->model->register($request['username'], $request["password"], $request['email']);

        if (!$account) {
            $fails[] = new FieldValidationFail("email", "L'email existe déjà");
        }

        if (!empty($fails)) {
            return ViewHttpResponse::twig("display_register.html.twig", ['fails' => $fails, 'password' => $requestData['password'], 'confirmpassword' => $requestData['confirmpassword'], 'accept' => true]);
        }
        $session->setAccount($account);

        $target_url = $session->getInitialTarget();
        if ($target_url != null) {
            return HttpResponse::redirectAbsolute($target_url);
        }

        return HttpResponse::redirect("/home");
    }


    public function displayLogin(): HttpResponse {
        return ViewHttpResponse::twig("display_login.html.twig", []);
    }

    /**
     * logins given account credentials
     * @param mixed[] $request
     * @param MutableSessionHandle $session
     * @return HttpResponse
     */
    public function login(array $request, MutableSessionHandle $session): HttpResponse {
        $fails = [];
        $account = $this->model->login($request['email'], $request['password'], $fails);
        if (!empty($fails)) {
            return ViewHttpResponse::twig("display_login.html.twig", ['fails' => $fails, 'password' => $request['password'], 'email' => $request['email']]);
        }

        $session->setAccount($account);

        $target_url = $session->getInitialTarget();
        $session->setInitialTarget(null);
        if ($target_url != null) {
            return HttpResponse::redirectAbsolute($target_url);
        }

        return HttpResponse::redirect("/home");
    }

}
